package eu.jonahbauer.raytracing.render.spectral.spectrum;

import eu.jonahbauer.raytracing.render.spectral.ColorMatchingFunction;
import org.jetbrains.annotations.NotNull;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UncheckedIOException;
import java.nio.charset.StandardCharsets;

public final class Spectra {
    private static final Spectrum X;
    private static final Spectrum Y;
    private static final Spectrum Z;

    static {
        var count = Spectrum.LAMBDA_MAX - Spectrum.LAMBDA_MIN + 1;
        var x = new double[count];
        var y = new double[count];
        var z = new double[count];

        try (
                var is = Spectra.class.getResourceAsStream("/eu/jonahbauer/raytracing/spectrum/CIE_xyz_1931_2deg.csv");
                var in = new BufferedReader(new InputStreamReader(is, StandardCharsets.US_ASCII))
        ) {
            for (int i = 0; i < count; i++) {
                var line = in.readLine().split(",");
                x[i] = Double.parseDouble(line[1]);
                y[i] = Double.parseDouble(line[2]);
                z[i] = Double.parseDouble(line[3]);
            }
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }

        X = new DenselySampledSpectrum(x, Spectrum.LAMBDA_MIN);
        Y = new DenselySampledSpectrum(y, Spectrum.LAMBDA_MIN);
        Z = new DenselySampledSpectrum(z, Spectrum.LAMBDA_MIN);
    }

    /**
     * {@return the X color matching curve}
     */
    public static @NotNull Spectrum X() {
        return X;
    }

    /**
     * {@return the Y color matching curve}
     */
    public static @NotNull Spectrum Y() {
        return Y;
    }

    /**
     * {@return the Z color matching curve}
     */
    public static @NotNull Spectrum Z() {
        return Z;
    }

    /**
     * {@return the D illuminant at the given temperature}
     * @param T the temperature
     */
    public static @NotNull Spectrum D(double T) {

    }

    public static @NotNull Spectrum getNamed(@NotNull String name) {

    }

    private Spectra() {
        throw new UnsupportedOperationException();
    }
}
